// ANSI/CEA 2034A SPINORAMA PLOT
// from a set of 70 CLIO .mls polar measurements 
// ---------------------------------------------
// 
// v1.00 - 12/04/2023
// dp@audiomatica.com
//

clear
funcprot(0);

// script variables
rootfilename='centrale'; 
octfrac=3;              // fraction of octave smoothing
npts=1024;              // log spaced frequency points between 20 Hz and 20 kHz to be calculated

// define functions
function [MLSsize, Fcamp, TimeW, TimeWb, TimeWe, frq, tim, frqdata, timdata]=readMLS(filename);
    [fd]=mopen(filename,'rb');
    skip=mget(28,'uc',fd);
    RelBackComp=mget(1,'ui',fd);
    if RelBackComp<627 then
        mclose(fd);
        error('File not compatible');
    end;
    skip=mget(765,'c',fd);
    //time window (0=no, 1=HalfHann, 2=Hann, 3=HalfBH, 4=BH)
    TimeW=mget(1,'c',fd);
    skip=mget(2,'c',fd);
    TimeWb=mget(1,'ui',fd) //first sample of selected impulse
    TimeWe=mget(1,'ui',fd) //last sample of selected impulse
    MLSsize=mget(1,'ui',fd) //MLS size
    skip=mget(3,'c',fd);
    //Y scale unit (3=Pascal, 5=Ohm, 0,1,2,4=Volts)
    ScaleType=mget(1,'c',fd)
    Fcamp=mget(1,'ui',fd); //sampling frequency
    skip=mget(136,'c',fd);
    MLSPRe=mget(MLSsize,'f',fd); //impulse data, real part
    MLSPIm=mget(MLSsize,'f',fd); //impulse data, imaginary part
    MLSXRe=mget(MLSsize,'f',fd); //frequency response data, real part
    MLSXIm=mget(MLSsize,'f',fd); //frequency response data, imag. part
    mclose(fd);
    frqdata=complex(MLSXRe,MLSXIm);
    timdata=complex(MLSPRe,MLSPIm);
    frq=0:Fcamp/MLSsize:(Fcamp/MLSsize)*(MLSsize-1);
    tim=0:1/Fcamp:(MLSsize-1)/Fcamp;
endfunction

function [outfrq,outresponse]=freqlintolog(frq,inresponse,fstart,fstop,npoints)
    fmult=(fstop/fstart)^(1/(npoints-1));
    outfrq=fmult.^(0:(npoints-1)).*fstart;
    d=splin(frq,inresponse);
    outresponse=interp(outfrq,frq,inresponse,d);
endfunction

function [outresponse]=smoothresponse(frq,inresponse,octfrac);
    frq1=frq.*2^(-1/octfrac/2);
    frq2=frq.*2^(1/octfrac/2);
    FRQ=repmat(frq,size(frq,2),1);
    FRQ1=repmat(frq1',1,size(frq1,2));
    FRQ2=repmat(frq2',1,size(frq2,2));
    COND=((FRQ<FRQ2) & (FRQ>=FRQ1));
    RESPONSE=repmat(inresponse,size(inresponse,2),1);
    PRESPONSE=10.^(RESPONSE/20);
    poutresponse=sum(PRESPONSE.*COND,2)./sum(COND,2);
    outresponse=20.*log10(poutresponse);
endfunction

// Sound Power Weighting Values from ANSI/CEA 2034A appendix C
// with 10 degrees resolution
wval=[
0.000604486
0.004730189
0.008955027
0.012387354
0.014989611
0.016868154
0.018165962
0.019006744
0.019477787
0.019629373
0.019477787
0.019006744
0.018165962
0.016868154
0.014989611
0.012387354
0.008955027
0.004730189
0.000604486
0.004730189
0.008955027
0.012387354
0.014989611
0.016868154
0.018165962
0.019006744
0.019477787
0.019629373
0.019477787
0.019006744
0.018165962
0.016868154
0.014989611
0.012387354
0.008955027
0.004730189
0.000604486
]'

// end functions ----------------------

// LOAD DATA
disp("Load data...");

// load H and V data 72 points at 10 degree resolution
// i=0 theta=-180, i=19 theta=0, i=37 theta=180
// load horizontal data (no symmetry)
for i=1:37
    filename=rootfilename+' 0 '+string(100*(i-19)*10)+'.mls';
    disp("Load "+filename);
    [MLSsize, Fcamp, TimeW, TimeWb, TimeWe, frq, tim, frqdata, timdata]=readMLS(filename);
    frqns=frq(2:MLSsize/2);
    respns=20*log10(abs(frqdata(2:MLSsize/2)));
    [outfrq,outresponse]=freqlintolog(frq,20*log10(abs(frqdata)),20,20000,npts);
    [outresponsesmooth]=smoothresponse(outfrq,outresponse,octfrac);
    horfrqdata(i,:)=outresponsesmooth;
end

// load vertical data (no symmetry)
for i=1:37
    filename=rootfilename+' 9000 '+string(100*(i-19)*10)+'.mls';
    disp("Load "+filename);
    [MLSsize, Fcamp, TimeW, TimeWb, TimeWe, frq, tim, frqdata, timdata]=readMLS(filename);
    frqns=frq(2:MLSsize/2);
    respns=20*log10(abs(frqdata(2:MLSsize/2)));
    [outfrq,outresponse]=freqlintolog(frq,20*log10(abs(frqdata)),20,20000,npts);
    [outresponsesmooth]=smoothresponse(outfrq,outresponse,octfrac);
    verfrqdata(i,:)=outresponsesmooth;
end

// ON AXIS response
disp("Calculate ON AXIS response...");

responseOA=horfrqdata(19,:);

// calculate LISTENING WINDOW curve
disp("Calculate LISTENING WINDOW response...");

// V 0, +10, -10
// H 0, +10, +20, +30, -10, -20, -30
hind=[18 19 20];
vind=[16 17 18 19 20 21 22];

appo=zeros(1,npts);
for i=1:size(hind,2)
    pappo=10.^(horfrqdata(hind(i),:)/20);
    appo=appo+pappo;
end
for i=1:size(vind,2)
    pappo=10.^(verfrqdata(vind(i),:)/20);
    appo=appo+pappo;
end
appo=appo./(size(hind,2)+size(vind,2));
responseLW=20.*log10(appo);

// calculate EARLY REFLECTIONS curve
disp("Calculate EARLY REFLECTIONS response...");

// V -20, -30, -40, 40, 50, 60
// H 0, +10, +20, +30, +40, +50, +60, +70, +80, +90, -10, -20, -30, -40, -50, -60, -70, -80, -90, 180
hind=[10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 37];
vind=[15 16 17 23 24 25];

appo=zeros(1,npts);
for i=1:size(hind,2)
    pappo=10.^(horfrqdata(hind(i),:)/20);
    appo=appo+pappo;
end
for i=1:size(vind,2)
    pappo=10.^(verfrqdata(vind(i),:)/20);
    appo=appo+pappo;
end
appo=appo./(size(hind,2)+size(vind,2));
responseER=20.*log10(appo);

// calculate VERTICAL REFLECTIONS curve
disp("Calculate VERTICAL REFLECTIONS response...");

// V -20, -30, -40, +40, +50, +60
vind=[15 16 17 23 24 25];

appo=zeros(1,npts);
for i=1:size(vind,2)
    pappo=10.^(verfrqdata(vind(i),:)/20);
    appo=appo+pappo;
end
appo=appo./(size(vind,2));
responseVR=20*log10(appo);

// calculate HORIZONTAL REFLECTIONS curve
disp("Calculate HORIZONTAL REFLECTIONS response...");

// H use all horizontal data in 10 degree steps
hind=2:37;

appo=zeros(1,npts);
for i=1:size(hind,2)
    pappo=10.^(horfrqdata(hind(i),:)/20);
    appo=appo+pappo;
end
appo=appo./(size(hind,2));
responseHR=20*log10(appo);

// calculate SOUND POWER curve
disp("Calculate SOUND POWER response...");

// using weigthed values according to table
hind=2:37;
vind=[2:18 20:36];

appo=zeros(1,npts);
for i=1:size(hind,2)
    pappo=wval(hind(i)).*10.^(horfrqdata(hind(i),:)/20);
    appo=appo+pappo;
end
for i=1:size(vind,2)
    pappo=wval(vind(i)).*10.^(verfrqdata(vind(i),:)/20);
    appo=appo+pappo;
end
//appo=appo./(size(hind,2)+size(vind,2));
responseSP=20*log10(appo);

// calculate SOUND POWER DIRECTIVITY INDEX (SPDI)
disp("Calculate DIRECTIVITY INDEXES...");

dirindSP=responseLW-responseSP;

// calculate EARLY REFLECTIONS DIRECTIVITY INDEX (ERDI)
dirindER=responseLW-responseER;

// plot all response curves
f1=scf()
f1.figure_size=[1024 768];
plot2d(outfrq,responseOA+94,2)
plot2d(outfrq,responseLW+94,3)
plot2d(outfrq,responseER+94,4)
plot2d(outfrq,responseVR+94,5)
plot2d(outfrq,responseHR+94,6)
plot2d(outfrq,responseSP+94,7)
a1=gca()
a1.log_flags="lnn"
a1.tight_limits="on";
a1.box="on";
a1.grid=[1 1];
a1.grid_style=[9 9]
ymax=ceil((max([responseLW responseER responseVR responseHR responseSP])+94)/10)*10;
a1.data_bounds=[20 ymax-50; 20000 ymax];
title("Spinorama Plot ANSI/CEA 2034A");
xlabel("frequency (Hz)")
ylabel("level (dBSPL)")
afrq=[20 30 40 50 60 70 80 90 100 200 300 400 500 600 700 800 900 1000 2000 3000 4000 5000 6000 7000 8000 9000 10000 20000]';
astr=["20" "" "" "50" "" "" "" "" "100" "200" "" "" "500" "" "" "" "" "1k" "2k" "" "" "5k" "" "" "" "" "10k" "20k"]';
a1.x_ticks = tlist(["ticks", "locations", "labels"], afrq, astr);
a1.sub_tics=[0 1];

// plot DI curves on secondary vertical axis
a2=newaxes()
// faux curves labels
plot2d(1:2,1:2,2);
plot2d(1:2,1:2,3);
plot2d(1:2,1:2,4);
plot2d(1:2,1:2,5);
plot2d(1:2,1:2,6);
plot2d(1:2,1:2,7);
plot2d(outfrq,dirindSP,2)
plot2d(outfrq,dirindER,5)
a2.filled = 'off';
a2.tight_limits = 'on';
a2.y_location="right"
a2.log_flags="lnn";
a2.data_bounds=[20 -5; 20000 45];
ylabel("directivity index (dB)")
a2.x_ticks = tlist(["ticks", "locations", "labels"], [ ]', [ ]');
a2.y_ticks = tlist(["ticks", "locations", "labels"], [-5 0 5 10 15]', ["-5" "0" "5" "10" "15"]');

// set curves colors and styles
cOA=a1.children(6).children(1); // ON AXIS RESPONSE
cLW=a1.children(5).children(1); // LISTENING WINDOW
cER=a1.children(4).children(1); // EARLY REFLECTIONS
cVR=a1.children(3).children(1); // VERTICAL REFLECTIONS
cHR=a1.children(2).children(1); // HORIZONTAL REFLECTIONS
cSP=a1.children(1).children(1); // SOUND POWER
cSPDI=a2.children(2).children(1); // SOUND POWER DI
cERDI=a2.children(1).children(1); // EARLY REFLECTIONS DI
cOA.foreground=2; 
cOA.line_style=1;
cLW.foreground=5; 
cLW.line_style=1;
cER.foreground=2; 
cER.line_style=3;
cVR.foreground=2; 
cVR.line_style=6;
cHR.foreground=2; 
cHR.line_style=8;
cSP.foreground=6; 
cSP.line_style=1;
cSPDI.foreground=6; 
cSPDI.line_style=3;
cERDI.foreground=2; 
cERUDI.line_style=4;

// set also for the legend
// this is an hack because apparently there is no other
// way to manage a legend with multiple axes
c2OA=a2.children(8).children(1); // ON AXIS RESPONSE
c2LW=a2.children(7).children(1); // LISTENING WINDOW
c2ER=a2.children(6).children(1); // EARLY REFLECTIONS
c2VR=a2.children(5).children(1); // VERTICAL REFLECTIONS
c2HR=a2.children(4).children(1); // HORIZONTAL REFLECTIONS
c2SP=a2.children(3).children(1); // SOUND POWER
c2OA.foreground=cOA.foreground;
c2OA.line_style=cOA.line_style;
c2LW.foreground=cLW.foreground;
c2LW.line_style=cLW.line_style;
c2ER.foreground=cER.foreground;
c2ER.line_style=cER.line_style;
c2VR.foreground=cVR.foreground;
c2VR.line_style=cVR.line_style;
c2HR.foreground=cHR.foreground;
c2HR.line_style=cHR.line_style;
c2SP.foreground=cSP.foreground;
c2SP.line_style=cSP.line_style;

// create legend
legend(["On Axis Response" "Listening Window" "Early Reflections" "Vertical Reflections" "Horizontal Reflections" "Sound Power" "Sound Power DI" "Early Reflections DI"]);
a2.children(1).legend_location="lower_caption";

// resize axes to the same dimensions to fit legend
a2.margins=[0.1 0.1 0.1 0.3];
a1.margins=a2.margins;
